## simulating data with given NCP
## ncp =s^2 * sum(slopes^2/ave.p)


# generate random points on ellipsoid sum(sc_i^2 * x_i^2)=1
rellipse <- function(n, scale){
    dim <- length(scale)
    u <- matrix(rnorm(n*dim), nrow=dim, ncol=n)
    ssu <- apply(u, 2, function(x)sum(x^2))
    z <- scale(u, center=FALSE, scale=sqrt(ssu))
    z <- diag(1/scale, nrow=dim) %*% z
    t(z)
}    

# generate random points on K-sphere sum(x_i^2)=1
# conditional on linear combination sum(a_i x_i)=0
rcondsphere <- function(n, dim, a=rep(1,dim)){
    amod <- head(a, -1) / tail(a, 1)
    A <- outer(amod, amod) + diag(1, nrow=dim-1)
    # eigenvalues of A are (1+sum(amod^2), 1, ...,1)
    eA <- eigen(A)
    S <- eA$vectors
    # note: S diag(eA$values) S' = A
    # S%*%Y is a hypershere
    z <- rellipse(n, scale=sqrt(eA$values))
    y <- z %*% t(S)
    y.K <- - y %*% amod
    res <- - cbind(y, y.K)
    res
}

# generate slopes to achieve given weighted squared-sum
# need sum(slopes)=0, sum(slopes^/ave.p)=S
rslope <- function(S, ave.p){
     # will generate z=slope*sqrt(S*ave.p)
    # sum(sqrt(ave.p*S)*z)=0, sum(z^2)=1
    a <- sqrt(ave.p * S)
    z <- rcondsphere(1, dim=length(ave.p), a=a)[1,]
    slopes <- z * a
    slopes
}    


# generate probability matrix with given ncp
gen.pmatrix.ncp <- function(ave.p, ncp, nc, nvec, maxit=100){
  #generate matrix with probability distribution of outcomes in each column
  # with linear slope of probabilities from baseline to end
    if (sum(ave.p) != 1) stop("Average probabilities should sum to 1")
    if (any(ave.p==0)) stop("Average probabilities shoud be all positive")

    nr <- length(ave.p)
    pmat <- matrix(ave.p, nrow=nr, ncol=nc, byrow=FALSE)

    if (ncp == 0) return(pmat)
    
    scores <- 1:nc
    N <- sum(nvec)
    cbar <- as.vector(crossprod(nvec, scores)) / N
    s2 <- sum(nvec * (scores-cbar)^2)
    S <- ncp/s2
    
    abs.slope.max <- pmin(ave.p, 1-ave.p) * 2/nc
    if (sum(abs.slope.max^2/ave.p) <  S) stop("Target ncp cannot be achieved")

    admissible <- FALSE
    iter <- 0
    while (!admissible && iter < maxit){
        iter <- iter+1
        sl <- rslope(S, ave.p)
        admissible <- all(abs(sl) < abs.slope.max)
    }
    if (iter >= maxit) stop("Target ncp was not achieved in maximum number of attempts")
    
    pmat <- pmat + sl * (col(pmat) - (nc+1)/2)  
    pmat
}

# generate random (average) probabilities
rand.probs <- function(k, ngrid=52){
  # generates k values with sum of 1
  # selecting from 'ngrid' equally spaced points (includes ends)
  idx <- sort(sample.int(ngrid-2, size = k-1, replace=FALSE))
  y <- seq(0, 1, length.out = ngrid)[idx+1]
  res <- diff(c(0, y, 1))
  res
}

# generate random matrix with given ncp
ran.pmatrix.ncp <- function(ncp, K, nc, nvec, ngrid=52, maxit=100){
    success <- FALSE
    iter <- 0
    while (!success && iter < maxit){
        iter <- iter + 1
        p <- rand.probs(k=K, ngrid=ngrid)
        pmat <- tryCatch(
            gen.pmatrix.ncp(ave.p=p, ncp=ncp, nc=nc, nvec=nvec, maxit=maxit),
            error=function(e)NULL)
        success <- !is.null(pmat)
    }
    if (iter >= maxit) stop("Target ncp was not achieved in maximum number of attempts")
    
    pmat
}
        
# generate random matrix with given ncp and L constant rows
ran.pmatrix0.ncp <- function(ncp, K, nc, nvec, L=0, ngrid=52, maxit=100, overall.ncp=TRUE){
    success <- FALSE
    iter <- 0
    while (!success && iter < maxit){
        iter <- iter + 1
        p <- rand.probs(k=K, ngrid=ngrid)
        p.null <- head(p, L)
        p.alt <- p[(L+1):K]
        p.alt.sum <- sum(p.alt)
        ncp.target <- if (overall.ncp) ncp/p.alt.sum else ncp
        pmat <- tryCatch(
            {p1 <- gen.pmatrix.ncp(ave.p=p.alt/p.alt.sum, ncp=ncp.target,
                                   nc=nc, nvec=nvec, maxit=maxit);
             p10 <- rbind(matrix(p.null, nrow=L, ncol=nc),
                          p1 * p.alt.sum)
             p10},                       
             error=function(e)NULL)
        success <- !is.null(pmat)
    }
    if (iter >= maxit) stop("Target ncp was not achieved in maximum number of attempts")
    
    pmat
}
